-- startup_chimes.lua
-- Simple mod to add startup chimes to all vanilla vehicles in BeamNG.drive

local M = {}

-- Sound file paths
local SOUND_PATH = "startup_chimes/sounds/"
local vehicleSounds = {
    -- Default fallback sounds
    ["default"] = {
        door = SOUND_PATH .. "default_door_chime.ogg",
        startup = SOUND_PATH .. "default_startup_chime.ogg"
    },
    
    -- Example: Different sounds for different trims
    ["covet_hatchback"] = {
        door = SOUND_PATH .. "hatchback_door_chime.ogg",
        startup = SOUND_PATH .. "hatchback_startup_chime.ogg"
    },
    
    ["etk800_sedan"] = {
        door = SOUND_PATH .. "sedan_door_chime.ogg",
        startup = SOUND_PATH .. "sedan_startup_chime.ogg"
    },
    
    ["gavril_pickup"] = {
        door = SOUND_PATH .. "pickup_door_chime.ogg",
        startup = SOUND_PATH .. "pickup_startup_chime.ogg"
    },
    
    ["vivace_hatchback"] = {
        door = SOUND_PATH .. "hatchback_door_chime.ogg", 
        startup = SOUND_PATH .. "hatchback_startup_chime.ogg"
   }
    -- Add more vehicle trims as needed
}

-- Function to get sound file for a vehicle
local function getSound(vehicleID, isStartupSound)
    local soundType = isStartupSound and "startup" or "door"
    
    -- Try to find exact match for vehicle ID
    for pattern, sounds in pairs(vehicleSounds) do
        if string.find(vehicleID, pattern) then
            return sounds[soundType]
        end
    end
    
    -- Fall back to default
    return vehicleSounds["default"][soundType]
end

-- Function to play door chime
local function playDoorChime(data)
    if not data or not data.vehicle then return end
    
    -- Check if door state changed to open
    if data.new and data.new == "open" and (data.old == "closed" or data.old == nil) then
        local vehicle = data.vehicle
        local vehicleID = vehicle:getID()
        local soundFile = getSound(vehicleID, false)
        
        Engine.Audio.playOnce(soundFile)
    end
end

-- Function to play startup chime
local function playStartupChime(data)
    if not data or not data.vehicle then return end
    
    -- Check if engine was just started
    if data.new and data.new == "on" and (data.old == "off" or data.old == nil) then
        local vehicle = data.vehicle
        local vehicleID = vehicle:getID()
        local soundFile = getSound(vehicleID, true)
        
        Engine.Audio.playOnce(soundFile)
    end
end

-- Initialize the mod for a vehicle
local function initForVehicle(vehicle)
    if not vehicle then return end
    
    -- Register door event listeners
    for i = 0, vehicle:getPartCount() - 1 do
        local part = vehicle:getPartByIndex(i)
        if part and part:getType() == "door" then
            part:registerAfterStateChange("startup_chimes.playDoorChime")
        end
    end
    
    -- Register engine event listener
    local engine = vehicle:getEngine()
    if engine then
        engine:registerAfterStateChange("startup_chimes.playStartupChime")
    end
end

-- Called when a vehicle spawns
local function onVehicleSpawned(vehicleID)
    local vehicle = be:getObjectByID(vehicleID)
    if vehicle then
        initForVehicle(vehicle)
    end
end

-- Initialize the mod
local function init()
    -- Register the onVehicleSpawned event
    registerCoreModule("startup_chimes")
    registerEvent("onVehicleSpawned", "startup_chimes.onVehicleSpawned")
    
    -- Expose functions to global space
    M.playDoorChime = playDoorChime
    M.playStartupChime = playStartupChime
    M.onVehicleSpawned = onVehicleSpawned
    
    log('I', 'startup_chimes', 'Startup Chimes mod initialized')
end

-- Clean up when mod is unloaded
local function destroy()
    unregisterCoreModule("startup_chimes")
    log('I', 'startup_chimes', 'Startup Chimes mod destroyed')
end

-- Expose module functions
M.init = init
M.destroy = destroy

return M